# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

project = "canonical-sphinx"
copyright = "2024, Canonical"
author = "Canonical"

extensions = [
    "canonical_sphinx",
]

html_context = {
    "product_page": "github.com/canonical/canonical-sphinx",
    "github_url": "https://github.com/canonical/canonical-sphinx",
}

github_username = "canonical"
github_repository = "canonical-sphinx"



# This file is part of canonical-sphinx.
#
# Copyright 2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Sphinx theme for Canonical documentation."""
from typing import Any, Dict
from pathlib import Path

from sphinx.application import Sphinx


def setup(app: Sphinx) -> Dict[str, Any]:
    """Add "canonical_sphinx_theme" as a valid html theme."""
    app.add_html_theme("canonical_sphinx_theme", str(Path(__file__).parent))
    return {
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



# This file is part of canonical-sphinx.
#
# Copyright 2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Basic canonical-sphinx package demo integration tests."""
import subprocess


def test_cli():
    expected = "Hello *craft team!\n"

    actual = subprocess.check_output(["canonical-sphinx-hello"], text=True)

    assert expected == actual



# Copyright 2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Integration tests for building documentation with the extension and theme."""
import shutil
import subprocess
from datetime import datetime
from pathlib import Path

import bs4
import pytest


@pytest.fixture
def example_project(request, tmp_path) -> Path:
    project_root = request.config.rootpath
    example_dir = project_root / "example"

    # Copy the project into the test's own temporary dir, to avoid clobbering
    # the sources.
    target_dir = tmp_path / "example"
    shutil.copytree(example_dir, target_dir)

    return target_dir


def test_canonical_sphinx(example_project):
    # Build the example project
    build_dir = example_project / "_build"
    subprocess.check_call(
        ["sphinx-build", "-b", "html", "-W", example_project, build_dir],
    )

    # Now verify the built artifacts

    # Check that static assets were copied over
    static_dir = build_dir / "_static"
    assets = ["example-css.css", "example-js.js", "example-tag.png"]
    for asset in assets:
        assert (static_dir / asset).is_file()

    # Check various fields/elements on the generated html
    index = build_dir / "index.html"
    soup = bs4.BeautifulSoup(index.read_text())

    # Document title
    assert soup.title.string == "Example Project Docs"

    # Copyright
    copyright_ = soup.find("div", {"class": "copyright"}).string.strip()
    year = datetime.today().year
    assert copyright_ == f"Copyright © {year}, Example Project Authors"

    # Logo
    logo = soup.find("a", {"class": "p-logo"})
    assert logo.attrs["href"] == "https://github.com/example/project"

    logo_img = logo.findChild("img")
    assert logo_img.attrs["src"] == "_static/example-tag.png"

    logo_text = logo.findChild("div").string.strip()
    assert logo_text == "Example Project"

    # Discourse link
    discourse_ref = soup.find(
        "a",
        {"href": "https://discourse.example-project.com"},
    ).string.strip()
    assert discourse_ref == "Forum"

    # Links to create issue and edit on Github
    github_issue = soup.find("div", {"class": "issue-github"}).findChild("a")
    assert github_issue["href"].startswith(
        "https://github.com/example/project/issues/new?",
    )
    assert github_issue.string.strip() == "Open a GitHub issue for this page"

    github_edit = soup.find("div", {"class": "edit-github"}).findChild("a")
    expected_edit = (
        "https://github.com/example/project/edit/v9.9.9/example/docs/index.rst"
    )
    assert github_edit["href"] == expected_edit
    assert github_edit.string.strip() == "Edit this page on GitHub"



# This file is part of canonical-sphinx.
#
# Copyright 2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Sphinx configuration, extension and theme for Canonical documentation."""
import importlib.util
import os
from pathlib import Path
from typing import Any, Dict

from sphinx.application import Sphinx


def setup(app: Sphinx) -> Dict[str, Any]:
    """Perform the main configuration and theme-setting."""
    # These are options that the user can set on their "conf.py"
    # (many options are still missing).
    app.add_config_value(
        "disable_feedback_button",
        default=False,
        rebuild="env",
        types=bool,
    )
    app.add_config_value("slug", default="", rebuild="env", types=str)

    extra_extensions = [
        "myst_parser",
    ]

    optional_packages = [
        "sphinx_design",
        "sphinx_tabs.tabs",
        "sphinx_reredirects",
        "canonical.youtube-links",
        "canonical.related-links",
        "canonical.custom-rst-roles",
        "canonical.terminal-output",
        "canonical.contributor-listing",
        "sphinx_copybutton",
        "sphinxext.opengraph",
        "sphinxcontrib.jquery",
        "notfound.extension",
    ]

    for package in optional_packages:
        try:
            if importlib.util.find_spec(package) is not None:
                extra_extensions.append(package)
            else:
                print(f"{package} not found.\n{package} will not be configured.")
        except ModuleNotFoundError:  # noqa: PERF203
            print(f"{package} not found.\n{package} will not be configured.")

    # These are the extra extensions that we need.

    for ext in extra_extensions:
        app.setup_extension(ext)

    # Hook into config-inited so we can do more work after "conf.py" is parsed.
    app.connect(  # pyright: ignore [reportUnknownMemberType]
        "config-inited",
        config_inited,
    )

    return {
        "version": "0.1",
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }


def config_inited(app: Sphinx, config: Any) -> None:  # noqa: ANN401
    """Read user-provided values and setup defaults."""
    config.myst_enable_extensions.update(["substitution", "deflist", "linkify"])

    config.exclude_patterns.extend(
        [
            "_build",
            "Thumbs.db",
            ".DS_Store",
            ".sphinx",
        ],
    )

    # The URL prefix for the notfound extension depends on whether the documentation uses versions.
    # For documentation on documentation.ubuntu.com, we also must add the slug.
    url_version = ""
    url_lang = ""

    # Determine if the URL uses versions and language
    if (
        "READTHEDOCS_CANONICAL_URL" in os.environ
        and os.environ["READTHEDOCS_CANONICAL_URL"]
    ):
        url_parts = os.environ["READTHEDOCS_CANONICAL_URL"].split("/")

        if (
            len(url_parts) >= 2  # noqa: PLR2004 (magic value)
            and "READTHEDOCS_VERSION" in os.environ
            and os.environ["READTHEDOCS_VERSION"] == url_parts[-2]
        ):
            url_version = url_parts[-2] + "/"

        if (
            len(url_parts) >= 3  # noqa: PLR2004 (magic value)
            and "READTHEDOCS_LANGUAGE" in os.environ
            and os.environ["READTHEDOCS_LANGUAGE"] == url_parts[-3]
        ):
            url_lang = url_parts[-3] + "/"

    # Set notfound_urls_prefix to the slug (if defined) and the version/language affix
    slug = config.slug
    if slug:
        notfound_urls_prefix = "/" + slug + "/" + url_lang + url_version
    elif len(url_lang + url_version) > 0:
        notfound_urls_prefix = "/" + url_lang + url_version
    else:
        notfound_urls_prefix = ""

    config.notfound_urls_prefix = notfound_urls_prefix

    config.html_theme = "canonical_sphinx_theme"
    config.html_last_updated_fmt = ""
    config.html_permalinks_icon = "¶"

    if config.html_title == "":
        config.html_theme_options = {"sidebar_hide_name": True}

    theme_dir = Path(__file__).parent / "theme"
    theme_assets = theme_dir / "static"

    config.html_static_path.append(str(theme_assets))

    html_favicon = theme_assets / "favicon.png"
    if not config.html_favicon:
        config.html_favicon = str(html_favicon)

    extra_css = [
        "custom.css",
        "header.css",
        "github_issue_links.css",
        "furo_colors.css",
    ]
    config.html_css_files.extend(extra_css)

    html_js_files = ["header-nav.js"]

    # Issue: We don't have access to the builder here yet
    # Setting templates_path for epub makes the build fail
    # if builder == "dirhtml" or builder == "html":
    config.templates_path.append(str(theme_dir / "templates"))
    config.notfound_template = "404.html"

    html_context = config.html_context

    disable_feedback_button = config.disable_feedback_button
    if html_context.get("github_issues") and not disable_feedback_button:
        html_js_files.append("github_issue_links.js")

    values_and_defaults = [
        ("product_tag", "_static/tag.png"),
        ("github_version", "main"),
        ("github_folder", "docs"),
        ("github_issues", "enabled"),
        ("discourse", "https://discourse.ubuntu.com"),
        ("sequential_nav", "none"),
        ("display_contributors", True),
    ]

    has_contributor_listing = "canonical.contributor-listing" in app.extensions

    for value, default in values_and_defaults:
        html_context.setdefault(value, default)

    html_context["has_contributor_listing"] = has_contributor_listing

    config.html_js_files.extend(html_js_files)



# Copyright 2023-2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
from unittest import mock

import canonical_sphinx


def test_plugin_registration():
    mocker = mock.Mock(canonical_sphinx.Sphinx)
    config = canonical_sphinx.setup(mocker)
    assert config["parallel_read_safe"] is True
    assert config["parallel_write_safe"] is True
    mocker.assert_has_calls([mock.call.setup_extension("canonical_sphinx.config")])


def test_theme_plugin_registration():
    from canonical_sphinx import theme

    mocker = mock.Mock(canonical_sphinx.Sphinx)
    with mock.patch.object(canonical_sphinx.theme, "__file__"):
        config = theme.setup(mocker)
        assert config["parallel_read_safe"] is True
        assert config["parallel_write_safe"] is True
        mocker.assert_has_calls(
            [
                mock.call.add_html_theme(
                    "canonical_sphinx_theme",
                    f"MagicMock{os.path.sep}__file__",
                ),
            ],
        )



# This file is part of canonical-sphinx.
#
# Copyright 2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Sphinx configuration, extension and theme for Canonical documentation."""
from typing import List, Optional, Any, Dict

from sphinx.application import Sphinx

try:
    from ._version import __version__
except ImportError:  # pragma: no cover
    from importlib.metadata import version, PackageNotFoundError

    try:
        __version__ = version("canonical-sphinx")
    except PackageNotFoundError:
        __version__ = "dev"


def hello(people: Optional[List[Any]] = None) -> None:
    """Says hello."""
    print("Hello *craft team!")
    if people:
        for person in people:
            print(f"Hello {person}!")


def setup(app: Sphinx) -> Dict[str, Any]:
    """Configure the main extension and theme."""
    app.setup_extension("canonical_sphinx.config")
    return {
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }


__all__ = ["__version__", "setup"]



# This file is part of starcraft.
#
# Copyright 2023-2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Basic Starcraft package demo unit tests."""
# pyright: reportFunctionMemberAccess=false
from unittest import mock

import canonical_sphinx


def test_version():
    assert canonical_sphinx.__version__ is not None


def test_hello(mocker):
    mocker.patch("builtins.print")

    canonical_sphinx.hello()

    print.assert_called_once_with("Hello *craft team!")


def test_hello_people(mocker):
    mocker.patch("builtins.print")

    canonical_sphinx.hello(["people"])

    print.assert_has_calls(
        [
            mock.call("Hello *craft team!"),
            mock.call("Hello people!"),
        ],
    )



# Copyright 2024 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Example project to showcase the extension's configuration."""
from datetime import datetime

############################################################
### Project information
############################################################

# Product name
project = "Example Project"
author = "Example Project Authors"

# The title you want to display for the documentation in the sidebar.
# You might want to include a version number here.
# To not display any title, set this option to an empty string.
html_title = project + " Docs"

# The default value uses the current year as the copyright year.
#
# For static works, it is common to provide the year of first publication.
# Another option is to give the first year and the current year
# for documentation that is often changed, e.g. 2022–2023 (note the en-dash).
#
# A way to check a GitHub repo's creation date is to obtain a classic GitHub
# token with 'repo' permissions here: https://github.com/settings/tokens
# Next, use 'curl' and 'jq' to extract the date from the GitHub API's output:
#
# curl -H 'Authorization: token <TOKEN>' \
#   -H 'Accept: application/vnd.github.v3.raw' \
#   https://api.github.com/repos/canonical/<REPO> | jq '.created_at'
copyright = f"{datetime.today().year}, {author}"

# Set the path to your own static assets here
html_static_path = ["_static"]

# (Some settings must be part of the html_context dictionary, while others
#  are on root level. Don't move the settings.)
html_context = {
    # Change to the link to the website of your product (without "https://")
    # For example: "ubuntu.com/lxd" or "microcloud.is"
    # If there is no product website, edit the header template to remove the
    # link (see the readme for instructions).
    "product_page": "github.com/example/project",
    # Add your product tag (the orange part of your logo, will be used in the
    # header) to ".sphinx/_static" and change the path here (start with "_static")
    # (default is the circle of friends)
    "product_tag": "_static/example-tag.png",
    # Change to the discourse instance you want to be able to link to
    # using the :discourse: metadata at the top of a file
    # (use an empty value if you don't want to link)
    "discourse": "https://discourse.example-project.com",
    # Change to the GitHub URL for your project
    "github_url": "https://github.com/example/project",
    # Change to the branch for this version of the documentation
    "github_version": "v9.9.9",
    # Change to the folder that contains the documentation
    # (usually "/" or "/docs/")
    "github_folder": "/example/docs/",
    # Change to an empty value if your GitHub repo doesn't have issues enabled.
    # This will disable the feedback button and the issue link in the footer.
    "github_issues": "enabled",
    # Controls the existence of Previous / Next buttons at the bottom of pages
    # Valid options: none, prev, next, both
    "sequential_nav": "none",
    # Controls whether to display the contributors for each file
    "display_contributors": False,
}

# If your project is on documentation.ubuntu.com, specify the project
# slug (for example, "lxd") here.
slug = "example_project"

############################################################
### Redirects
############################################################

# Set up redirects (https://documatt.gitlab.io/sphinx-reredirects/usage.html)
# For example: 'explanation/old-name.html': '../how-to/prettify.html',

redirects = {}

############################################################
### Link checker exceptions
############################################################

# Links to ignore when checking links

linkcheck_ignore = ["www.example.com"]

# Pages on which to ignore anchors
# (This list will be appended to linkcheck_anchors_ignore_for_url)

custom_linkcheck_anchors_ignore_for_url = []

############################################################
### Additions to default configuration
############################################################

## The following settings are appended to the default configuration.
## Use them to extend the default functionality.

# Add extensions
extensions = [
    "canonical_sphinx",
]

# Add MyST extensions
myst_extensions = []

# Add files or directories that should be excluded from processing.
exclude_patterns = [
    "example-exclude*",
]

# Add CSS files (located in .sphinx/_static/)
html_css_files = ["example-css.css"]

# Add JavaScript files (located in .sphinx/_static/)
html_js_files = ["example-js.js"]

## The following settings override the default configuration.

# By default, the documentation includes a feedback button at the top.
# You can disable it by setting the following configuration to True.
disable_feedback_button = False

# Add tags that you want to use for conditional inclusion of text
# (https://www.sphinx-doc.org/en/master/usage/restructuredtext/directives.html#tags)
tags = []

github_username = "example"
github_repository = "project"
